# Simulation Experiments for Section 7.2.
# "Varying the Number of Outliers at a Fixed Location."
# Signal in four covariates
# Compare NLMINB, Coordinate Descent C code, MLE

args=(commandArgs(TRUE))

if (length(args)==0) {
  print("No arguments supplied.")
  ##supply default values
  nc = 0
  NOI = 50
} else {
  for (i in 1:length(args)) {
    eval(parse(text=args[[i]]))
  }
}

Wdir = getwd()
Rdir = "../../../Rfunctions"
setwd(Rdir)
source("rLogistic.R")
source("genLogit.R")
source("mean.l2e.R")
setwd(Wdir)

# Parameters for my data.
n = 100
p = 4
shift = 3
ss = 0.25
x.sd = 0.4
beta.true = matrix(c(0,1,0.5,1,2), ncol=1)
w = 1
w.opt = 1-nc/(2*n+nc)

noi = 20000
seed = 111908
savefile=paste("comparisonNC=",nc,"shift=",shift,".rda",sep="")

set.seed(seed)
seed.seq = sample(1:111908, NOI)

CD = matrix(NA, p+1, NOI)
NLMIN = matrix(NA, p+1, NOI)
CD.wopt = matrix(NA, p+1, NOI)
NLMIN.wopt = matrix(NA, p+1, NOI)
MLE = matrix(NA, p+1, NOI)

for (iter in 1:NOI) {

	data = genLogit(n=n, p=p, nc=nc, shift=shift, ss=ss, x.sd=x.sd,
		beta=beta.true, seed=seed.seq[iter])
	X = data$X
	Y = data$Y

	Xc = cbind(1,apply(X[,-1,drop=F],2,FUN=function(x){x-mean(x)}))
	X.cmean = apply(X[,-1,drop=F],2,mean)
	
#-- Coordinate descent

	system.time({ll2E =  call_logisticL2E(Xc[,-1,drop=F], Y, alpha=0, lambda=0, beta0=1, beta=matrix(1,p,1),niter=noi,w=w)})
	CD[,iter] = ll2E

        system.time({ll2E =  call_logisticL2E(Xc[,-1,drop=F], Y, alpha=0, lambda=0, beta0=1, beta=matrix(1,p,1),niter=noi,w=w.opt)})
	CD.wopt[,iter] = ll2E

#-- Mean L2E
	theta0 = c(w,matrix(0, nrow=p+1, ncol=1))
	theta0 = c(w,beta.true)
	upper = c(w, rep(Inf, p+1)); lower = c(w, rep(-Inf, p+1))
	system.time({opt.mean.l2e = nlminb(start=theta0, mean.l2e, X=Xc, Y=Y, lambda=0, alpha=1,
		lower=lower, upper=upper)})
	NLMIN[,iter] = opt.mean.l2e$par[-1]

        theta0 = c(w.opt,matrix(0, nrow=p+1, ncol=1))
	theta0 = c(w.opt,beta.true)
	upper = c(w.opt, rep(Inf, p+1)); lower = c(w.opt, rep(-Inf, p+1))
	system.time({opt.mean.l2e = nlminb(start=theta0, mean.l2e, X=Xc, Y=Y, lambda=0, alpha=1,
		lower=lower, upper=upper)})
	NLMIN.wopt[,iter] = opt.mean.l2e$par[-1]

#-- MLE
	system.time({mle = glm(Y ~ ., family=binomial(logit), data=data.frame(Y=Y, X=Xc[,-1,drop=F]))})
	MLE[,iter] = mle$coef
}

CD[1,] = CD[1,] - X.cmean %*% CD[-1,,drop=F]
NLMIN[1,] = NLMIN[1,] - X.cmean %*% NLMIN[-1,,drop=F]
CD.wopt[1,] = CD.wopt[1,] - X.cmean %*% CD.wopt[-1,,drop=F]
NLMIN.wopt[1,] = NLMIN.wopt[1,] - X.cmean %*% NLMIN.wopt[-1,,drop=F]
MLE[1,] = MLE[1,] - X.cmean %*% MLE[-1,,drop=F]

save(w, NLMIN, CD, NLMIN.wopt, CD.wopt, MLE, beta.true, file=savefile)
