# Script to create Figure 4.
library(ggplot2)
library(xtable)
files = dir(pattern=".rda")

for (i in 1:length(files)) {
	load(files[i])
	nc = unlist(strsplit(unlist(strsplit(files[i],"NC="))[2],"shift"))[1]
	l2e.wopt = cbind(apply(CD.wopt,1,mean),apply(CD.wopt,1,sd))
	l2e.w1 = cbind(apply(CD,1,mean),apply(CD,1,sd))
	mle = cbind(apply(MLE,1,mean),apply(MLE,1,sd))
	print(paste("*********** NC=",nc," ***********",sep=""))
	print(xtable(cbind(mle,l2e.w1),digits=3))
}

normData = data.frame()
for (i in 1:length(files)) {
	load(files[i])
	nc = unlist(strsplit(unlist(strsplit(files[i],"NC="))[2],"shift"))[1]
	CD.norm = apply(as.matrix(CD[-1,,drop=F]),2,FUN=function(x){sqrt(sum(x^2))})
	temp.df = data.frame(norm = CD.norm)
	temp.df$nc = nc
	temp.df$method = "l2ew=1"
	normData = rbind(normData, temp.df)	
	MLE.norm = apply(as.matrix(MLE[-1,,drop=F]),2,FUN=function(x){sqrt(sum(x^2))})
	temp.df = data.frame(norm = MLE.norm)
	temp.df$nc = nc
	temp.df$method = "mle"
	normData = rbind(normData, temp.df)	
}

normData$method = factor(normData$method, levels=c("mle","l2ew=1"), ordered=TRUE)
limits = c("mle","l2ew=1")
labels = c(expression(paste("MLE")), expression(paste(L[2],"E")))

normData$nc = ordered(normData$nc, levels=c(0,1,5,10,15,20))
q = ggplot(data=normData, aes(x=nc, y=norm))
q = q + geom_boxplot(aes(fill=method)) + theme_bw()
q = q + scale_fill_manual(name="Method", values=c("#1F78B4","#B2DF8A"), breaks=limits, labels=labels)
q = q + xlab("Number of Outliers") + ylab(expression(paste("2-norm of ", beta)))
q = q + ylim(c(0,max(normData$norm)))
#q + geom_hline(yintercept=sqrt(sum(beta.true[-1]^2)))
q + opts(axis.title.x=theme_text(vjust=-0.25))
ggsave("normVsNumberOutliers.pdf", width=8, height=3)
