# Run this script to create Figures 5 and 6, after running crossValidationExp.R
#  as it requires the data file: VariableSelectionExperiment_10fold.rda

library("ggplot2")
library("Matrix")

load("VariableSelectionExperiment_10fold.rda")

nReps = length(MLE.TP)

####### Robust CV Error curves

u.l2e = apply(L2E.CV,2,median)
md.l2e = apply(L2E.CV,2,mad)
minIX.l2e = max(which(u.l2e == min(u.l2e)))
oneMAD.l2e = max(which(u.l2e <= u.l2e[minIX.l2e] + md.l2e[minIX.l2e]))
lb.l2e = u.l2e - md.l2e
ub.l2e = u.l2e + md.l2e
lambda.l2e = -log(L2E_lambda_HX[[nReps]])

df.l2e = data.frame(lambda = lambda.l2e, u = u.l2e, ub = u.l2e + md.l2e, lb =
u.l2e-md.l2e)
df.l2e$Method = "L2E"

u.mle = apply(MLE.CV,2,median)
md.mle = apply(MLE.CV,2,mad)
minIX.mle = min(which(u.mle == min(u.mle)))
oneMAD.mle = min(which(u.mle <= u.mle[minIX.mle] + md.mle[minIX.mle]))
lb.mle = u.mle - md.mle
ub.mle = u.mle + md.mle
lambda.mle = -log(MLE_lambda_HX[[nReps]])

df.mle = data.frame(lambda = lambda.mle, u = u.mle, ub = u.mle + md.mle, lb =
u.mle-md.mle)
df.mle$Method = "MLE"

u.hhsvm = apply(HHSVM.CV,2,median)
md.hhsvm = apply(HHSVM.CV,2,mad)
minIX.hhsvm = min(which(u.hhsvm == min(u.hhsvm)))
oneMAD.hhsvm = min(which(u.hhsvm <= u.hhsvm[minIX.hhsvm] + md.hhsvm[minIX.hhsvm]))
lb.hhsvm = u.hhsvm - md.hhsvm
ub.hhsvm = u.hhsvm + md.hhsvm
lambda.hhsvm = -log(HHSVM_lambda_HX[[nReps]])

df.hhsvm = data.frame(lambda = lambda.hhsvm, u = u.hhsvm, ub = u.hhsvm + md.hhsvm, lb =
u.hhsvm-md.hhsvm)
df.hhsvm$Method = "HHSVM"

df = rbind(df.l2e, df.mle)
df = rbind(df, df.hhsvm)


df$Method = factor(df$Method, levels=c("L2E","MLE","HHSVM"), labels=c(expression(paste(L[2],"E")),"MLE","HHSVM"), ordered=TRUE)

vline1.dat <- data.frame(Method = unique(df$Method), vl=c(lambda.l2e[minIX.l2e],lambda.mle[minIX.mle],lambda.hhsvm[minIX.hhsvm]))
vline2.dat <- data.frame(Method = unique(df$Method), vl=c(lambda.l2e[oneMAD.l2e],lambda.mle[oneMAD.mle],lambda.hhsvm[oneMAD.hhsvm]))


q = ggplot(data=df, aes(x = lambda, y=u, ymin=lb, ymax=ub))
q = q + geom_pointrange(size=0.375)
q = q + scale_x_continuous(expression(paste("-log(",lambda,")",sep="")))
q = q + geom_vline(aes(xintercept=vl), data=vline1.dat, colour=rgb(27/255, 158/255, 119/255), linetype=6,size=0.75)
q = q + geom_vline(aes(xintercept=vl), data=vline2.dat, colour=rgb(217/255, 95/255, 2/255), linetype=5,size=0.75)
q = q + facet_grid(.~Method, scales="free", labeller=label_parsed)
q + theme_bw() + ylab(label="Robust CV Error") + opts(axis.title.x=theme_text(vjust=-0.25))
ggsave(filename="robust_CV.pdf", width=8, height=3)

####################### Plot Regularization paths
p.true = 50
iSet = 1
ixSNP = list(); colours = list()
for (i in 1:p.true) {
	ixSNP[[i]]	 = i
	colours[[i]] = rgb(215/255, 25/255, 28/255)
}

rLogistic.fit = L2E_models[[nReps]][[iSet]]
data=data.frame(beta = t(as.matrix(rLogistic.fit$beta)), lambda=lambda.l2e)
melted = melt(data, id.vars=c("lambda"))

vars = matrix(NA,p.true,1)
for (i in 1:p.true) {
  vars[i] = paste("beta.",i,sep="")
}
  ix = which(melted$variable %in% vars)
	melted$truth = 0
	melted$truth[ix] = 1
	melted$truth = factor(melted$truth, levels=c(0,1))

l2e.m_noise = melted[-ix,]
l2e.m_true = melted[ix,]
l2e.m_noise$Method = "L2E"
l2e.m_true$Method = "L2E"

glmnet.fit = MLE_models[[nReps]][[iSet]]

model_sizes = apply(glmnet.fit$beta,2,FUN=function(x) {sum(abs(sign(x)))})
cutOff = max(which(model_sizes <= MLE.TP[nReps]+MLE.FP[nReps])) + 5
subSample = 1:cutOff

data=data.frame(beta = t(as.matrix(glmnet.fit$beta[,subSample])), lambda=lambda.mle[subSample])
melted = melt(data, id.vars=c("lambda"))

vars = matrix(NA,p.true,1)
for (i in 1:p.true) {
  vars[i] = paste("beta.V",i,sep="")
}
	ix = which(melted$variable %in% vars)
	melted$truth = 0
	melted$truth[ix] = 1
	melted$truth = factor(melted$truth, levels=c(0,1))

mle.m_noise = melted[-ix,]
mle.m_true = melted[ix,]
mle.m_noise$Method = "MLE"
mle.m_true$Method = "MLE"

hhsvm.fit = HHSVM_models[[nReps]][[iSet]]

data=data.frame(beta = t(as.matrix(hhsvm.fit$beta)), lambda=lambda.hhsvm)
melted = melt(data, id.vars=c("lambda"))

vars = matrix(NA,p.true,1)
for (i in 1:p.true) {
  vars[i] = paste("beta.",i,sep="")
}
	ix = which(melted$variable %in% vars)
	melted$truth = 0
	melted$truth[ix] = 1
	melted$truth = factor(melted$truth, levels=c(0,1))

hhsvm.m_noise = melted[-ix,]
hhsvm.m_true = melted[ix,]
hhsvm.m_noise$Method = "HHSVM"
hhsvm.m_true$Method = "HHSVM"

m_noise = rbind(l2e.m_noise, mle.m_noise)
m_noise = rbind(m_noise, hhsvm.m_noise)
m_true = rbind(l2e.m_true, mle.m_true)
m_true = rbind(m_true, hhsvm.m_true)

m_noise$Method = factor(m_noise$Method, levels=c("L2E","MLE","HHSVM"), labels=c(expression(paste(L[2],"E")),"MLE","HHSVM"), ordered=TRUE)
m_true$Method = factor(m_true$Method, levels=c("L2E","MLE","HHSVM"), labels=c(expression(paste(L[2],"E")),"MLE","HHSVM"), ordered=TRUE)

red_colour = rgb(215/255, 25/255, 28/255)

q = ggplot(data=m_noise, aes(x=lambda, y=value, group=variable)) + geom_line(aes(colour=truth),alpha=0.4,size=0.25)  + theme_bw()
q = q +geom_line(data=m_true, aes(colour=truth),alpha=0.95,size=0.5)
q = q + scale_colour_manual(values=c("black",red_colour))
q = q + ylab("Coefficients") + xlab(expression(paste("-log(",lambda,")",sep="")))
q = q + facet_grid(.~Method, scales="free", labeller=label_parsed)
q = q + opts(legend.position="none", axis.title.x=theme_text(vjust=-0.25))
q 
ggsave(filename="regularization_paths.pdf", width=8, height=3)
